"""
Main entry point for running Modpack Changelogger as a module.

This allows the package to be imported in other Python scripts.
"""

from modpack_changelogger.cli_wrapper import main

if __name__ == "__main__":
    main()
